!---------------------------------------------------------------------------------------
! PREP_SYNDATA: prep_syndata
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> prep_syndata 
!!
!! Purpose:    Prepares synthetic data (fluxes and observations) for use with 
!!             FLEXINVERT.
!!
!! Details:    Uses the input, i.e. error covariances and modelled mixing ratios from
!!             an existing FLEXINVERT run (forward run only is sufficient). The
!!             prior fluxes in this run serve as the "truth" and a new prior is 
!!             generated by adding random error to the fluxes. The modelled mixing 
!!             ratios serve as the observations and also have added random error.
!!             (See also mod_perturb.f90)
!!
!---------------------------------------------------------------------------------------

program prep_syndata

  use mod_var
  use mod_settings
  use mod_perturb
  use mod_save

  implicit none

  ! set types
  type (config_t)                         :: config
  type (files_t)                          :: files

  character(max_path_len)                 :: settings_files
  character(max_path_len)                 :: settings_config
  real, dimension(:,:,:), allocatable     :: flx
  real, dimension(:), allocatable         :: state
  real, dimension(:), allocatable         :: obs, obserr

  call getarg(1,settings_files)
  if (settings_files == '') then
    stop 'ERROR: need to specify SETTINGS_files file'
  endif

  call getarg(2,settings_config)
  if (settings_files == '') then
    stop 'ERROR: need to specify SETTINGS_config file'
  endif

  write(logid,*) 'Using: '//trim(settings_files)
  write(logid,*) 'Using: '//trim(settings_config)

  ! initialization
  ! --------------

  ! read file settings
  call read_file_settings(settings_files, files)

  ! read config settings
  call read_config_settings(settings_config, config)

  ! initialize parameters
  call initialize(files, config)

  ! get prior error covariance 
  call get_error_cov(files)
  
  ! get prior fluxes
  allocate( flx(nxgrid,nygrid,nt_flx))
  call get_flux(config, files, flx)

  ! get observations
  allocate( obs(maxobs) )
  allocate( obserr(maxobs) )
  call get_obs(config, files, obs, obserr) 

  ! perturbation
  ! ------------

  ! perturb fluxes
  allocate( state(nvar) )
  state(:) = 0.
  call perturb_flux(config, files, state)

  ! perturb obs
   call perturb_obs(config, files, obs, obserr)

  ! save output
  ! -----------

  call save_obs(config, files, obs, obserr)
  call save_flux(config, files, flx, state)
  close(logid)

end program



