!---------------------------------------------------------------------------------------
! FLEXINVERT: init_cini
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> init_cini
!!
!! Purpose:    Calculates the initial concentration for each observation using the
!!             4D initial concentration fields and the flexpart grid_initial files.
!!
!! Interface:
!!
!!    Inputs
!!             files   -  file data structure
!!             obs     -  observation data structure 
!!
!!    Externals
!!             caldate
!!             read_init              
!!             get_initconc
!!
!---------------------------------------------------------------------------------------

subroutine init_cini(files, obs)

  use mod_flexpart
  use mod_settings
  use mod_strings
  use mod_var
  use mod_dates
  use mod_obs
  use mod_tools

  implicit none

  type (files_t),        intent (in)     :: files
  type (obs_t),          intent (in out) :: obs

  character(max_path_len)                :: path_flexrec, filename
  character(max_name_len)                :: varname
  character(len=8)                       :: adate, areldate, areltime
  logical                                :: lexist
  integer                                :: lastjjjjmm, prejjjjmm, projjjjmm
  integer                                :: jjjjmm, jjjjmmdd, hhmiss, mm, eomday
  integer                                :: ix, jy, kz, i, n
  real                                   :: conc
  real(kind=8)                           :: jdmid, jdpre, jdlast
  real(kind=8), dimension(nobs)          :: jdates
  real, dimension(nobs,ncini)            :: cini
  integer, dimension(nobs)               :: ind
  real, dimension(nxgrid,nygrid,nzgrid)  :: gridini
  real, dimension(nxgrid,nygrid,nzgrid)  :: concini, preconcini, proconcini


  ! loop over observations
  ! ----------------------

  lastjjjjmm = 0
  concini(:,:,:) = 0.
  preconcini(:,:,:) = 0.
  proconcini(:,:,:) = 0.
  cini(:,:) = 0.

  ! sort obs chronologically for efficiency
  jdates = obs%jdate(:)
  call sort(nobs,jdates,ind)
!  print*, 'init_cini: ind = ',ind
!  print*, 'init_cini: jdates = ',jdates
!  print*, 'init_cini: obs%jdate = ',obs%jdate

  do i = 1, nobs

    ! define file name
    call caldate(jdates(i), jjjjmmdd, hhmiss)
    write(adate,'(I6.6)') jjjjmmdd/100
    write(areldate,'(I8.8)') jjjjmmdd
    write(areltime,'(I6.6)') hhmiss
    path_flexrec = trim(files%path_flexpart)//trim(obs%recs(ind(i)))//'/'//trim(adate)//'/'
    filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
!    print*, 'init_cini: filename = ',filename

    ! read flexpart initial concentrations weighting
    inquire(file=trim(filename),exist=lexist)
    if(lexist) then
      call read_init(filename, gridini)
      ! convert from ppt to fraction of one
      gridini = gridini*1.e-12
    else
      write(logid,*) 'WARNING: cannot find ',filename
      go to 10
    endif

    ! calculate termination time of trajectory
    call caldate(jdates(i) - trajdays, jjjjmmdd, hhmiss)
    jjjjmm = jjjjmmdd/100
   
    ! preceding and proceding months
    mm = jjjjmm - (jjjjmm/100)*100
    if( mm.gt.1 ) then
      prejjjjmm = jjjjmm - 1
    else
      prejjjjmm = (jjjjmm/100 - 1)*100 + 12
    endif
    if( mm.lt.12 ) then
      projjjjmm = jjjjmm + 1
    else
      projjjjmm = (jjjjmm/100 + 1)*100 + 1
    endif

    jdmid = juldate(jjjjmm*100+14, 0)
    jdpre = juldate(prejjjjmm*100+14, 0)
    jdlast = juldate(lastjjjjmm*100+14, 0) 
    eomday = calceomday(prejjjjmm)

    ! get new concentrations if not same month as previous observation 
    if( jjjjmm.ne.lastjjjjmm ) then
      if( (lastjjjjmm.ne.0).and.(jdmid.eq.(jdlast+dble(eomday))) ) then
        ! this is proceeding month
        preconcini = concini
        concini = proconcini
        ! get proceeding concentration field
        write(adate,'(I6.6)') projjjjmm
        filename = str_replace(files%file_initconc, 'YYYYMM', adate)  
        filename = trim(files%path_initconc)//trim(filename)
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(filename, varname, proconcini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          go to 10
        endif
      else 
        ! get current concentration field
        write(adate,'(I6.6)') jjjjmm
        filename = str_replace(files%file_initconc, 'YYYYMM', adate)
        filename = trim(files%path_initconc)//trim(filename)
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(filename, varname, concini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          go to 10
        endif
        ! get previous concentration field
        write(adate,'(I6.6)') prejjjjmm
        filename = str_replace(files%file_initconc, 'YYYYMM', adate)  
        filename = trim(files%path_initconc)//trim(filename)
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(filename, varname, preconcini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          go to 10
        endif
        ! get next concentration field
        write(adate,'(I6.6)') projjjjmm
        filename = str_replace(files%file_initconc, 'YYYYMM', adate)  
        filename = trim(files%path_initconc)//trim(filename)
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(filename, varname, proconcini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          go to 10
        endif
      endif   
    endif

    ! calculate contribution of initial concentration to receptor
    do kz = 1, nzgrid
      do jy = 1, nygrid
        n = 0
        do while(.true.)
          n = n + 1
          if ( gbl_lat(jy).lt.cini_lat(n) ) exit
        end do
!        print*, 'init_cini: gbl_lat(jy), n = ',gbl_lat(jy), n
        do ix = 1, nxgrid
          ! interpolate between concentrations
          if( (jdates(i) - trajdays).lt.jdmid ) then
            conc = preconcini(ix,jy,kz) + (concini(ix,jy,kz) - preconcini(ix,jy,kz)) * &
                     real((jdates(i) - trajdays) - jdpre, kind=4)/30.
          else
            conc = concini(ix,jy,kz) + (proconcini(ix,jy,kz) - concini(ix,jy,kz)) * &
                     real((jdates(i) - trajdays) - jdmid, kind=4)/30.
          endif
          cini(i,n) = cini(i,n) + gridini(ix,jy,kz)*conc
        end do
      end do
    end do

10  continue

    lastjjjjmm = jjjjmm

  end do

  ! reorder cini for observations
!  print*, 'init_cini: ind = ',ind
!  print*, 'init_cini: cini = ',cini
  do i = 1, nobs
    obs%cini(ind(i),:) = cini(i,:)
  end do
!  print*, 'init_cini: obs%cini = ',obs%cini

end subroutine init_cini


