!---------------------------------------------------------------------------------------
! FLEXINVERT: initialize
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> initialize
!! 
!! Purpose:    Initializes variables needed for the inversion
!!
!! Interface:
!!
!!    Inputs
!!             files   -  file data structure
!!             config  -  configuration settings data structure
!!
!!    Externals
!!             read_header
!!             read_reclist
!!
!---------------------------------------------------------------------------------------

subroutine initialize(files, config)

  use mod_var
  use mod_settings
  use mod_dates
  use mod_flexpart, only: read_header

  implicit none

  type (files_t),  intent(in)            :: files
  type (config_t), intent(in)            :: config
  
  character(max_path_len)                :: filename
  logical                                :: lexist
  integer                                :: yyyymm
  character(len=6)                       :: adate
  character(len=2)                       :: aiter
  integer                                :: numpoint
  integer                                :: ibdate, ibtime
  integer, dimension(maxpoint)           :: releases
  real                                   :: bndx, bndy, delx, dely
  integer                                :: numx, numy, xshift
  integer                                :: i, ierr

  ! initialize log file
  ! -------------------

  if ( config%restart.eq.1 ) then
    open(logid,file=trim(files%path_output)//trim(files%file_log),status='old',action='write',access='append',iostat=ierr)
    write(logid,*) 'Picking-up run here...'
  else
    open(logid,file=trim(files%path_output)//trim(files%file_log),status='replace',action='write',iostat=ierr)
    write(logid,*) '***********************************************************'
    write(logid,*) '*                                                         *'
    write(logid,*) '*                   FLEXINVERTplus V0                     *'
    write(logid,*) '*                                                         *'
    write(logid,*) '***********************************************************'
    write(logid,*) ' '
  endif

  ! check if this is a restart
  ! --------------------------

  if ( (config%restart.eq.1).and.(config%method.eq.'congrad'.or.config%method.eq.'m1qn3') ) then
    ! restart crashed run
    lexist = .true.
    i = -1
    do while (lexist)
      i = i + 1
      write(aiter,fmt='(I2.2)') i
      inquire(file=trim(files%path_output)//'grad_'//aiter//'.txt',exist=lexist)
      write(logid,*) 'initialize: reading '//trim(files%path_output)//'grad_'//aiter//'.txt'
    end do
    lastiter = i - 1
    if ( lastiter.lt.0 ) then
      write(logid,*) 'ERROR initialize: grad for restart not found'
      stop
    endif
    write(logid,*) 'Restarting from iteration: ',lastiter
  endif

  ! initialize obsread file
  ! ----------------------- 

  open(200,file=trim(files%path_output)//'obsread.txt',action='write',status='replace',iostat=ierr)
  close(200)

  ! read receptor list file
  ! -----------------------

  filename = trim(files%file_recept)
  inquire(file=trim(filename),exist=lexist)
  if (.not.lexist) then
    write(logid,*) 'ERROR: cannot find '//trim(filename)
    stop
  endif
  write(logid,*) 'Reading receptor list: '//trim(filename)
  call read_reclist(filename) 

  ! initialize flexpart variables
  ! -----------------------------

  ! read header for global output
  yyyymm=config%datei/100
  write(adate,'(i6)') yyyymm
  lexist = .false.
  i = 1
  do while ( (.not.lexist).and.(i.le.nrec) )
    filename = trim(files%path_flexpart)//trim(recname(i))//'/'//adate//'/header'
    inquire(file=trim(filename),exist=lexist)
    i = i + 1
  end do
  if ( .not.lexist ) then
    write(logid,*) 'ERROR initialize: cannot find flexpart header'
    stop
  endif
  write(logid,*) 'Reading flexpart header: '//trim(filename)
  call read_header(filename, numpoint, ibdate, ibtime, releases, & 
                     numx, numy, bndx, bndy, delx, dely, xshift)
  ! global domain variables
  nxgrid = numx
  nygrid = numy
  llx = bndx
  lly = bndy
  dx = delx
  dy = dely
  nxshift = xshift
  if ( .not.allocated(gbl_lon) ) allocate( gbl_lon(nxgrid) )
  if ( .not.allocated(gbl_lat) ) allocate( gbl_lat(nygrid) )
  do i = 1, nxgrid
    gbl_lon(i) = llx + (i-1)*dx
  end do
  do i = 1, nygrid
    gbl_lat(i) = lly + (i-1)*dy
  end do
  
  ! read header for nested output
  if ( config%nested ) then
    lexist = .false.
    i = 1
    do while ( (.not.lexist).and.(i.le.nrec) )
      filename = trim(files%path_flexpart)//trim(recname(i))//'/'//adate//'/header_nest'
      inquire(file=trim(filename),exist=lexist)
      i = i + 1
    end do
    if ( .not.lexist ) then
      write(logid,*) 'ERROR initialize: cannot find flexpart header_nest'
      stop
    endif
    write(logid,*) 'Reading flexpart header_nest: '//trim(filename)
    call read_header(filename, numpoint, ibdate, ibtime, releases, & 
                       numx, numy, bndx, bndy, delx, dely, xshift)
    ! nested domain variables
    nnxgrid = numx
    nnygrid = numy
    nllx = bndx
    nlly = bndy
    ndx = delx
    ndy = dely
    nnxshift = xshift
    if ( .not.allocated(nest_lon) ) allocate( nest_lon(nnxgrid) )
    if ( .not.allocated(nest_lat) ) allocate( nest_lat(nnygrid) )
    do i = 1, nnxgrid
      nest_lon(i) = nllx + (i-1)*ndx
    end do
    do i = 1, nnygrid
      nest_lat(i) = nlly + (i-1)*ndy
    end do
  endif

  ! nzgrid is z dimension of grid_init files (read from header)
  ! nzlev is z dimension of grid_time files with surface only
  nzlev = 1 

  ! initialize date variables
  ! -------------------------

  ! julian dates for inversion interval
  juldatei = juldate(config%datei, 0)
  juldatef = juldate(config%datef, 0)

  ! number days in inversion interval
  nday = int(juldatef - juldatei + 1.)

  ! number of NEE fields per day
  nd_nee = config%num_nee_day

  ! number of initial mixing ratio time steps to optimize
  ntcini = max(1, nday/cinires)

  if ( config%spec.eq.'co2' ) then
    ! statres is time interval over which state variables are averaged
    ! statres_hr is the sub-daily time interval for state variables
    ! ndt is number of time steps per 24h in prior NEE
    ! flxtres is temporal resolution of low frequency fluxes (biomass burning and ocean)
    ! ntflx is number of time steps of low frequency fluxes
    ! ntstate is number of averaged time invervals for state variables
    statres = config%statres
    statres_hr = config%statres_hr
    ndt = int(24./statres_hr)
    flxtres = aint(365./real(config%nt_flx))
    ntflx =  nint(real(nday)/flxtres)
    ntstate = nint(real(nday)/statres)
  else
    ! statres is time resolution of state variables
    ! flxtres is temporal resolution of fluxes (same as state variables for non-CO2 tracers)
    ! ntflx is number of state time invervals
    ! ntstate is also number of state time invervals 
    statres = config%statres
    flxtres = config%statres
    ntflx = nint(real(nday)/statres)
    ntstate = ntflx
    ndt = 1
  endif
  if ( ntflx.lt.1 ) then
    write(logid,*) 'ERROR initialize: number flux time steps < 1 -> increase time interval'
    stop
  endif
  if ( ntstate.lt.1 ) then
    write(logid,*) 'ERROR initialize: state time steps < 1 -> increase time interval'
    stop
  endif

  ! initialize grid variables
  ! -------------------------
 
  rllx = config%w_edge_lon
  rlly = config%s_edge_lat
  rurx = config%e_edge_lon
  rury = config%n_edge_lat
  rdx = config%xres
  rdy = config%yres
  nxregrid = int((rurx - rllx)/rdx) 
  nyregrid = int((rury - rlly)/rdy)
  maxngrid = trajdays*ndgrid+2
  write(logid,*), 'maxngrid = ',maxngrid

  ! check region boundaries
  ! -----------------------

  if ( rllx.lt.llx ) then
    write(logid,*) 'ERROR: western boundary outside flexpart domain'
    stop
  endif
  if ( rurx.gt.(llx+nxgrid*dx) ) then
    write(logid,*) 'ERROR: eastern boundary outside flexpart domain'
    stop
  endif
  if ( rlly.lt.(lly) ) then
    write(logid,*) 'ERROR: southern boundary outside flexpart domain'
    stop
  endif
  if ( rury.gt.(lly+nygrid*dy) ) then
    write(logid,*) 'ERROR: northern boundary outside flexpart domain'
    stop
  endif
  
  if ( config%nested ) then
    if ( rllx.lt.nllx ) then
      write(logid,*) 'ERROR: western boundary outside flexpart nested domain'
      stop
    endif
    if ( rurx.gt.(nllx+nnxgrid*ndx) ) then
      write(logid,*) 'ERROR: eastern boundary outside flexpart nested domain'
      stop
    endif
    if ( rlly.lt.nlly ) then
      write(logid,*) 'ERROR: southern boundary outside flexpart nested domain'
      stop
    endif
    if ( rury.gt.(nlly+nnygrid*ndy) ) then
      write(logid,*) 'ERROR: northern boundary outside flexpart nested domain'
      stop
    endif
    if ( (ndx.ne.rdx).or.(ndy.ne.rdy) ) then
      write(logid,*) 'ERROR: nested flexpart resolution does not match inversion'
      stop
    endif
  else
    if ( (dx.ne.rdx).or.(dy.ne.rdy) ) then
      write(logid,*) 'ERROR: flexpart resolution does not match inversion'
      stop
    endif
  endif

  ! initialize region variables
  ! ---------------------------

  allocate( nbox_xy(nxregrid,nyregrid) )
  allocate( lsm_box(nxregrid,nyregrid) )
  nbox_xy(:,:) = 0
  lsm_box(:,:) = 0

  ! initial other variables
  ! -----------------------

  maxiter = config%maxiter

  write(logid,*) 'nday: ',nday
  write(logid,*) 'statres: ',statres
  write(logid,*) 'flxtres: ',flxtres
  write(logid,*) 'ntstate: ',ntstate
  write(logid,*) 'ntflx: ',ntflx
  if ( config%spec.eq.'co2' ) then
    write(logid,*) 'nd_nee: ',nd_nee
    write(logid,*) 'ndt: ',ndt
  endif
  write(logid,*) 'Regional domain:'
  write(logid,*) 'w_edge_lon: ',rllx
  write(logid,*) 's_edge_lat: ',rlly
  write(logid,*) 'e_edge_lon: ',rurx
  write(logid,*) 'n_edge_lat: ',rury
  write(logid,*) 'xres:',rdx
  write(logid,*) 'yres:',rdy
  write(logid,*) 'nxregrid: ',nxregrid
  write(logid,*) 'nyregrid: ',nyregrid
  write(logid,*) 'nzgrid: ',nzgrid
  write(logid,*) 'Flexpart global domain:'
  write(logid,*) 'llx: ',llx
  write(logid,*) 'lly: ',lly
  write(logid,*) 'dx: ',dx
  write(logid,*) 'dy: ',dy
  write(logid,*) 'nxgrid: ',nxgrid
  write(logid,*) 'nygrid: ',nygrid
  if ( config%nested ) then
    write(logid,*) 'Flexpart nested domain:'
    write(logid,*) 'nllx: ',nllx
    write(logid,*) 'nlly: ',nlly
    write(logid,*) 'ndx: ',ndx
    write(logid,*) 'ndy: ',ndy
    write(logid,*) 'nnxgrid: ',nnxgrid
    write(logid,*) 'nnygrid: ',nnygrid
  endif 
  write(logid,*) 'Length of backtrajectory in days: ',trajdays
  write(logid,*) 'Number of footprints per day: ',ndgrid
  write(logid,*) 'nxshift:',nxshift
  write(logid,*) 'Height of first flexpart layer: ',outheight(1)
  write(logid,*) 'Max number of footprints each release: ',maxngrid

end subroutine initialize


