!---------------------------------------------------------------------------------------
! FLEXINVERT: mod_perturb 
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_perturb
!!
!! Purpose:    Calculates the perturbations to the prior state and observation vectors.
!!
!! Details:    Perturbs the state and observation vector according to the error 
!!             characteristics of the B and R error covariance matrices, respectively.
!!   
!!             For the state vector:
!!             x = xb + B^(1/2)*r
!!             where:
!!               xb = prior/background value of x
!!               B  = prior error covariance matrix
!!               r  = random vector of mean zero and SD one 
!!
!!             For the observation vector:
!!             y = y + R^(1/2)*r
!!
!---------------------------------------------------------------------------------------

module mod_perturb

  use mod_var
  use mod_states
  use mod_covar

  implicit none
  private

  public :: perturb_state, perturb_obs, randnum

  contains

  ! --------------------------------------------------
  ! perturb_state
  ! --------------------------------------------------

  subroutine perturb_state(config, states, covar)

    use mod_var
    use mod_settings
    use mod_states
    use mod_covar

    type (config_t),       intent (in)       :: config
    type (states_t),       intent (in out)   :: states
    type (covar_t),        intent (in)       :: covar

    real(kind=8), dimension(neig)            :: tmp
    real(kind=8), dimension(ndvar)           :: tmp1
    real(kind=8), dimension(nvar)            :: perturb
    real, dimension(nvar)                    :: rvec
    integer                                  :: seed
    integer                                  :: i, j, k, it, jt

    ! random number vector
    seed = config%seed
    call randnum(nvar, seed, rvec)

    ! perturb = B^(1/2)r 
    perturb(:) = 0d0
    do jt = 1, ntstate
      do k = 1, ndvar
        do j = 1, neig
          tmp(j) = covar%evecs(k,j)*sqrt(covar%evals(j))
        end do
        do i = 1, ndvar
          tmp1(i) = dot_product(tmp,covar%evecs(i,:))
        end do
        do it = 1, ntstate
          perturb((jt-1)*ndvar+k) = perturb((jt-1)*ndvar+k) + covar%sqcort(jt,it) * &
                                       dot_product(tmp1,dble(rvec((it-1)*ndvar+1:it*ndvar)))
        end do
      end do
    end do
    if ( config%opt_cini ) then
      perturb(npvar+1:nvar) = config%cinierr * rvec(npvar+1:nvar)
    endif

    ! add perturbation to state vector 
    states%px0(:) = states%px0(:) + real(perturb(:),kind=4)
    if ( config%prior_bg.ne.1 ) then 
      states%px(:) = states%px0(:) 
    endif

  end subroutine perturb_state

  ! --------------------------------------------------
  ! perturb_obs
  ! --------------------------------------------------

  subroutine perturb_obs(config, obs)

    use mod_var
    use mod_settings
    use mod_obs

    type (config_t),       intent (in)       :: config
    type (obs_t),          intent (in out)   :: obs

    real, dimension(nobs)                    :: perturb
    real, dimension(nobs)                    :: rvec
    integer                                  :: i, seed

    ! random number vector
    seed = config%seed
    call randnum(nobs, seed, rvec)

    ! perturb = R^(1/2)r
    perturb(:) = 0.
    do i = 1, nobs
      perturb(i) = obs%err(i)*rvec(i)
    end do
 
    ! obs = obs + perturb
    obs%conc = obs%conc + perturb

  end subroutine perturb_obs

  ! --------------------------------------------------
  ! randnum
  ! --------------------------------------------------

  subroutine randnum(num, seed, rvec)

    integer,               intent (in)     :: num
    integer,               intent (in)     :: seed
    real, dimension(num),  intent (in out) :: rvec

    integer                                :: ssize, i
    real                                   :: ri
    integer, dimension(:), allocatable     :: seedvec

    ! generate vector of random numbers
    call random_seed
    call random_seed(size=ssize)
    allocate(seedvec(ssize))
    do i = 1, ssize
      seedvec(i) = seed
    end do
    call random_seed(put=seedvec(1:ssize))
    do i = 1, num
      call random_number(ri)
      rvec(i) = ri
    end do
    deallocate(seedvec)

    ! normalize random numbers
    rvec = rvec*num/sum(rvec)
 
    ! mean of zero
    rvec = rvec - 1.

  end subroutine randnum

  ! --------------------------------------------------


end module mod_perturb



